/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CaScadeS, a stylesheet editor for Composer.
 *
 * The Initial Developer of the Original Code is
 * Daniel Glazman.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Original author: Daniel Glazman <daniel@glazman.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//Cancel() is in EdDialogCommon.js
var insertNew = true;
var tagname = "TAG NAME"
var gElement    = null; // handle to actual element edited
var objectsArray;
var gTimerID;

const SHEET        = 1;
const STYLE_RULE   = 2;
const IMPORT_RULE  = 3;
const MEDIA_RULE   = 4;
const CHARSET_RULE = 5;
const PAGE_RULE    = 6;
const OWNER_NODE   = 7;

const GENERAL_TAB    = 1;
const TEXT_TAB       = 2;
const BACKGROUND_TAB = 3;
const BORDER_TAB     = 4;
const BOX_TAB        = 5;
const AURAL_TAB      = 6;

const GENERIC_SELECTOR      = 0;
const TYPE_ELEMENT_SELECTOR = 1;
const CLASS_SELECTOR        = 2;

const urlString = "url(";

var gHaveDocumentUrl = false;
var predefSelector = "";

function doDump(text, value) {
  dump("===> " + text + " : " + value + "\n");
}

// dialog initialization code
function Startup()
{
  if (!InitEditorShell())
    return;

  // gDialog is declared in EdDialogCommon.js
  // Set commonly-used widgets like this:
  gDialog.selectedTab = TEXT_TAB;
  gDialog.sheetsTreechildren            = document.getElementById("stylesheetsTree");
  gDialog.sheetsTree                    = document.getElementById("sheetsTree");
  gDialog.sheetInfoTab                  = document.getElementById("sheetInfoTab");
  gDialog.atimportButton                = document.getElementById("atimportButton");
  gDialog.atmediaButton                 = document.getElementById("atmediaButton");
  gDialog.linkButton                    = document.getElementById("linkButton");
  gDialog.styleButton                   = document.getElementById("styleButton");
  gDialog.ruleButton                    = document.getElementById("ruleButton");
  gDialog.removeButton                  = document.getElementById("removeButton");
  gDialog.upButton                      = document.getElementById("upButton");
  gDialog.downButton                    = document.getElementById("downButton");

  gDialog.selectedTab = GENERAL_TAB;
  gDialog.sheetInfoTabPanelTitle        = document.getElementById("sheetInfoTabPanelTitle");
  gDialog.textTab                       = document.getElementById("textTab");
  gDialog.brownFoxLabel                 = document.getElementById("brownFoxLabel");
  gDialog.backgroundImageInput          = document.getElementById("backgroundImageInput");
  gDialog.backgroundPreview             = document.getElementById("backgroundPreview");
  gDialog.sheetTabbox                   = document.getElementById("sheetTabbox");
  gDialog.backgroundColorInput          = document.getElementById("backgroundColorInput");
  gDialog.textColorInput                = document.getElementById("textColorInput");
  gDialog.backgroundRepeatMenulist      = document.getElementById("backgroundRepeatMenulist");
  gDialog.backgroundAttachmentCheckbox  = document.getElementById("backgroundAttachmentCheckbox");
  gDialog.xBackgroundPositionRadiogroup = document.getElementById("xBackgroundPositionRadiogroup");
  gDialog.yBackgroundPositionRadiogroup = document.getElementById("yBackgroundPositionRadiogroup");
  gDialog.fontFamilyRadiogroup          = document.getElementById("fontFamilyRadiogroup");
  gDialog.customFontFamilyInput         = document.getElementById("customFontFamilyInput");
  gDialog.predefFontFamilyMenulist      = document.getElementById("predefFontFamilyMenulist");
  gDialog.fontSizeInput                 = document.getElementById("fontSizeInput");

  gDialog.topBorderStyleMenulist        = document.getElementById("topBorderStyleMenulist");
  gDialog.topBorderWidthInput           = document.getElementById("topBorderWidthInput");
  gDialog.topBorderColorInput           = document.getElementById("topBorderColorInput");

  gDialog.leftBorderStyleMenulist       = document.getElementById("leftBorderStyleMenulist");
  gDialog.leftBorderWidthInput          = document.getElementById("leftBorderWidthInput");
  gDialog.leftBorderColorInput          = document.getElementById("leftBorderColorInput");

  gDialog.rightBorderStyleMenulist      = document.getElementById("rightBorderStyleMenulist");
  gDialog.rightBorderWidthInput         = document.getElementById("rightBorderWidthInput");
  gDialog.rightBorderColorInput         = document.getElementById("rightBorderColorInput");

  gDialog.bottomBorderStyleMenulist     = document.getElementById("bottomBorderStyleMenulist");
  gDialog.bottomBorderWidthInput        = document.getElementById("bottomBorderWidthInput");
  gDialog.bottomBorderColorInput        = document.getElementById("bottomBorderColorInput");

  gDialog.allFourBordersSame            = document.getElementById("allFourBordersSame");
  gDialog.borderPreview                 = document.getElementById("borderPreview");

  gDialog.volumeScrollbar               = document.getElementById("volumeScrollbar");
  gDialog.volumeMenulist                = document.getElementById("volumeMenulist");
  gDialog.muteVolumeCheckbox            = document.getElementById("muteVolumeCheckbox");

  gDialog.expertMode = true;
  gDialog.modified = false;
  gDialog.selectedIndex = -1;

  gHaveDocumentUrl = GetDocumentBaseUrl();

  InitDialog();
  
  // Set window location relative to parent window (based on persisted attributes)
  SetWindowLocation();

  // Set focus to first widget in dialog, e.g.:
}

function InitDialog() 
{
  // Initialize all dialog widgets here,
  // e.g., get attributes from an element for property dialog
  InitSheetsTree(gDialog.sheetsTreechildren);
}

function toggleExpertMode()
{
  gDialog.expertMode = !gDialog.expertMode;
  if (gDialog.expertMode && gDialog.selectedIndex == -1) {
    UpdateButtons(false, false, true, true, false, false);
  }
  else if (gDialog.expertMode && gDialog.selectedIndex != -1) {
    var external  = objectsArray[gDialog.selectedIndex].external;
    var type      = objectsArray[gDialog.selectedIndex].type;
    UpdateButtons(!external, !external, true, true, !external, (!external || (type == SHEET)) );
  }
  else {
    UpdateButtons(!gDialog.atimportButton.hasAttribute("disabled"),
                  !gDialog.atmediaButton.hasAttribute("disabled"),
                  !gDialog.linkButton.hasAttribute("disabled"),
                  !gDialog.styleButton.hasAttribute("disabled"),
                  !gDialog.ruleButton.hasAttribute("disabled"),
                  !gDialog.removeButton.hasAttribute("disabled"));
  }
}

function FlushChanges()
{
  if (gDialog.modified) {
    editorShell.editor.incrementModificationCount(1);
  }
  // Validate all user data and set attributes and possibly insert new element here
  // If there's an error the user must correct, return false to keep dialog open.
  for (var i = 0; i < objectsArray.length; i++) {
         
    if (objectsArray[i].modified && !objectsArray[i].external &&
        objectsArray[i].type == SHEET) {
      /* let's serialize this stylesheet ! */
      var sheet     = objectsArray[i].cssElt;
      SerializeEmbeddedSheet(sheet);
    }
  }
  SaveWindowLocation();
  return true; // do close the window
}

function InitSizeInput(size, property, menulistID, inputID)
{
  var menulist = document.getElementById(menulistID);
  var inputZone = document.getElementById(inputID);
  ClearMenulist(menulist);
  var pxItem      = AppendStringToMenulist(menulist, "px");
  var percentItem = AppendStringToMenulist(menulist, "%");
  var ptItem      = AppendStringToMenulist(menulist, "pt");
  var cmItem      = AppendStringToMenulist(menulist, "cm");
  var inItem      = AppendStringToMenulist(menulist, "in");
  var mmItem      = AppendStringToMenulist(menulist, "mm");
  var pcItem      = AppendStringToMenulist(menulist, "pc");
  var emItem      = AppendStringToMenulist(menulist, "em");
  var exItem      = AppendStringToMenulist(menulist, "ex");

  var res = size.match( /(\d*)(\D*)/ );
  var value = RegExp.$1;
  var unit = RegExp.$2;
  var selectedItem;
  if (unit == "px") selectedItem = pxItem;
  else if (unit == "%")  selectedItem = percentItem;
  else if (unit == "mm") selectedItem = mmItem;

  menulist.selectedItem = selectedItem;
  menulist.setAttribute("style", "width: 6em");
  inputZone.setAttribute("value", value);
}

function CleanSheetsTree(sheetsTree)
{
  var elt = sheetsTree.firstChild;
  while (elt) {
    var tmp = elt.nextSibling;
    sheetsTree.removeChild(elt);
    elt = tmp;
  }
}

function InitSheetsTree(sheetsTree)
{
  CleanSheetsTree(sheetsTree);
  var sst = editorShell.editorDocument.styleSheets;
  var o;
  if (sst) {
    var ssn = sst.length;
    objectsArray = new Array();
    if (ssn) {
      var i;
      for (i=0; i<ssn; i++) {
        var treeitem  = document.createElementNS(XUL_NS, "treeitem");
        var treerow   = document.createElementNS(XUL_NS, "treerow");
        var treecell  = document.createElementNS(XUL_NS, "treecell");
        
        var ownerTag = sst[i].ownerNode.nodeName.toLowerCase();
        var external = false;
        if (ownerTag == "style") {
          treecell.setAttribute("label", "internal stylesheet");
        }
        else if (ownerTag == "link") {
          treecell.setAttribute("label", sst[i].href);
          external = true;
        }
        o = newObject( treeitem, external, SHEET, sst[i], false, 0 );
        objectsArray.push(o);

        treerow.appendChild(treecell);
        treeitem.appendChild(treerow);
        treeitem.setAttribute("container", "true");
        var rules = sst[i].cssRules;
        AddRulesToTreechildren(treeitem, rules, external, 1);
        sheetsTree.appendChild(treeitem);
      }
    }
  }
}

function newObject( xulElt, external, type, cssElt, modified, depth)
{
  return {xulElt:xulElt,external:external,type:type,cssElt:cssElt,modified:modified,depth:depth};
}

function AddRulesToTreechildren(treeItem, rules, external, depth)
{
  if (rules.length) {
    var subtreechildren = document.createElementNS(XUL_NS, "treechildren");
    var j, o;
    var subtreeitem, subtreerow, subtreecell;
    for (j=0; j< rules.length; j++) {
      switch (rules[j].type) {
        case CSSRule.STYLE_RULE:
          subtreeitem  = document.createElementNS(XUL_NS, "treeitem");
          subtreerow   = document.createElementNS(XUL_NS, "treerow");
          subtreecell  = document.createElementNS(XUL_NS, "treecell");
          subtreecell.setAttribute("label", rules[j].selectorText);
          o = newObject( subtreeitem, external, STYLE_RULE, rules[j], false, depth );
          objectsArray.push(o);
          if (external) {
            subtreecell.setAttribute("properties", "external");
          }
          subtreerow.appendChild(subtreecell);
          subtreeitem.appendChild(subtreerow);
          subtreechildren.appendChild(subtreeitem);
          break;
        case CSSRule.IMPORT_RULE:
          subtreeitem  = document.createElementNS(XUL_NS, "treeitem");
          subtreerow   = document.createElementNS(XUL_NS, "treerow");
          subtreecell  = document.createElementNS(XUL_NS, "treecell");
          subtreecell.setAttribute("label", "@import "+rules[j].href, external);
          o = newObject( subtreeitem, external, IMPORT_RULE, rules[j], false, depth );
          objectsArray.push(o);
          if (external) {
            subtreecell.setAttribute("properties", "external");
          }
          subtreerow.appendChild(subtreecell);
          subtreeitem.appendChild(subtreerow);
          subtreeitem.setAttribute("container", "true");
          if (rules[j].styleSheet)
            AddRulesToTreechildren(subtreeitem, rules[j].styleSheet.cssRules, true, depth+1);
          subtreechildren.appendChild(subtreeitem);
          break;
        case CSSRule.MEDIA_RULE:
          subtreeitem  = document.createElementNS(XUL_NS, "treeitem");
          subtreerow   = document.createElementNS(XUL_NS, "treerow");
          subtreecell  = document.createElementNS(XUL_NS, "treecell");
          subtreecell.setAttribute("label", "@media "+rules[j].media.mediaText, external);
          o = newObject( subtreeitem, external, MEDIA_RULE, rules[j], false, depth );
          objectsArray.push(o);
          if (external) {
            subtreecell.setAttribute("properties", "external");
          }
          subtreerow.appendChild(subtreecell);
          subtreeitem.appendChild(subtreerow);
          subtreeitem.setAttribute("container", "true");
          AddRulesToTreechildren(subtreeitem, rules[j].cssRules, external, depth+1);
          subtreechildren.appendChild(subtreeitem);
          break;
        case CSSRule.CHARSET_RULE:
          subtreeitem  = document.createElementNS(XUL_NS, "treeitem");
          subtreerow   = document.createElementNS(XUL_NS, "treerow");
          subtreecell  = document.createElementNS(XUL_NS, "treecell");
          subtreecell.setAttribute("label", "@charset "+rules[j].encoding, external);
          o = newObject( subtreeitem, external, CHARSET_RULE, rules[j], false, depth );
          objectsArray.push(o);
          if (external) {
            subtreecell.setAttribute("properties", "external");
          }
          subtreerow.appendChild(subtreecell);
          subtreeitem.appendChild(subtreerow);
          subtreechildren.appendChild(subtreeitem);
          break;          
      }
    }
    treeItem.appendChild(subtreechildren);
  }
}

function onSelectCSSTreeItem(tab)
{
  if      (tab == "general")     tab = GENERAL_TAB;
  else if (tab == "text")        tab = TEXT_TAB;
  else if (tab == "background")  tab = BACKGROUND_TAB;
  else if (tab == "border")      tab = BORDER_TAB;
  else if (tab == "box")         tab = BOX_TAB;
  else if (tab == "aural")       tab = AURAL_TAB;
  
  var selectedItem =  getSelectedItem(gDialog.sheetsTree);
  gDialog.selectedIndex  = -1;
  gDialog.selectedObject = null;
  var i, l = objectsArray.length;
  for (i=0; i<l; i++) {
    if (objectsArray[i].xulElt == selectedItem) {
      gDialog.selectedIndex  = i;
      gDialog.treeItem       = objectsArray[i].xulElt;
      gDialog.externalObject = objectsArray[i].external;
      gDialog.selectedType   = objectsArray[i].type;
      gDialog.selectedObject = objectsArray[i].cssElt;
      gDialog.modifiedObject = objectsArray[i].modified;
      gDialog.depthObject    = objectsArray[i].depth;
      break;
    }
  }
  if (gDialog.selectedIndex == -1) {
    gDialog.sheetTabbox.selectedTab = gDialog.sheetInfoTab;
    return;
  }

  var external  = objectsArray[gDialog.selectedIndex].external;
  var type      = objectsArray[gDialog.selectedIndex].type;
  var cssObject = gDialog.selectedObject;
  UpdateButtons(!external, !external, true, true, !external, (!external || (type == SHEET)) );

  if (gDialog.selectedType != STYLE_RULE)
    tab = GENERAL_TAB;
  if (!tab)
    tab = gDialog.selectedTab ? gDialog.selectedTab : GENERAL_TAB;
  switch (tab) {
    case TEXT_TAB:
      gDialog.brownFoxLabel.removeAttribute("style");      
      InitTextTabPanel();
      return;
      break;
    case BACKGROUND_TAB:
      InitBackgroundTabPanel();
      return;
      break;
    case BORDER_TAB:
      InitBorderTabPanel();
      return;
      break;
    case BOX_TAB:
      InitBoxTabPanel();
      return;
      break;
    case AURAL_TAB:
      InitAuralTab();
      return;
      break;
  }
  gDialog.selectedTab = GENERAL_TAB;

  gDialog.sheetTabbox.selectedTab = gDialog.sheetInfoTab;
  
  var gridrows = document.getElementById("sheetInfoTabGridRows");
  var grid     = document.getElementById("sheetInfoTabGrid");

  if (gridrows) {
    gridrows.parentNode.removeChild(gridrows);
  }

  gridrows = document.createElementNS(XUL_NS, "rows");
  gridrows.setAttribute("id", "sheetInfoTabGridRows");
  grid.appendChild(gridrows);

  switch (type) {
    case OWNER_NODE:
      if (gDialog.selectedObject.sheet) {
        var index = gDialog.selectedIndex;
        Restart();
        selectTreeItem(objectsArray[index].xulElt);
        onSelectCSSTreeItem(tab);
        return;
      }
      break;
    case SHEET:
      grid.removeAttribute("style");
      var alternate = "";
      if (external &&
          cssObject.ownerNode.getAttribute("rel").toLowerCase() == "alternate stylesheet") {
        alternate = " (alternate)";
      }
      gDialog.sheetInfoTabPanelTitle.setAttribute("value", "Stylesheet"+alternate);
      AddLabelToInfobox(gridrows, "Type:", cssObject.type, false);
      AddCheckboxToInfobox(gridrows, "Disabled:", "check to disable stylesheet (cannot be saved)",
                           cssObject.disabled, "onStylesheetDisabledChange");
      var href;
      if (external) {
        href = cssObject.href;
      }
      else {
        href = "none (embedded into the document)";
      }
      AddLabelToInfobox(gridrows, "URL:", href, false);
      var mediaList = cssObject.media.mediaText;
      if (!mediaList || mediaList == "") {
        mediaList = "all";
      }
      AddEditableZoneToInfobox(gridrows, "Media list:", mediaList, "onStylesheetMediaChange", true);
      if (cssObject.title) {
        AddEditableZoneToInfobox(gridrows, "Title:", cssObject.title, "onStylesheetTitleChange", false);
      }
      if (cssObject.cssRules) {
        AddLabelToInfobox(gridrows, "Number of rules:", cssObject.cssRules.length, false);
      }
      break;
    case STYLE_RULE:
      var h = document.defaultView.getComputedStyle(grid.parentNode, "").getPropertyValue("height");
      grid.setAttribute("style", "overflow: auto; height: "+h);
      gDialog.sheetInfoTabPanelTitle.setAttribute("value", "Style rule");

      AddLabelToInfobox(gridrows, "Selector:", cssObject.selectorText, false);
      if (cssObject.style.length) {
        AddDeclarationToInfobox(gridrows, "Declarations:", GetDeclarationText(cssObject, 0),
                                GetDeclarationImportance(cssObject, 0), false);
        for (i=1; i<cssObject.style.length; i++) {
          AddDeclarationToInfobox(gridrows, "", GetDeclarationText(cssObject, i),
                                  GetDeclarationImportance(cssObject, i), false);
        }
      }
      break;
    case MEDIA_RULE:
      gDialog.sheetInfoTabPanelTitle.setAttribute("value", "Media rule");
      AddLabelToInfobox(gridrows, "Media list:", cssObject.media.mediaText, false);
      AddLabelToInfobox(gridrows, "Number of declarations:", cssObject.cssRules.length, false);
      break;
    case IMPORT_RULE:
      gDialog.sheetInfoTabPanelTitle.setAttribute("value", "Import rule");
      AddLabelToInfobox(gridrows, "URL:", cssObject.href, false);
      AddLabelToInfobox(gridrows, "Media list:", cssObject.media.mediaText, false);
      break;
  }
}

function UpdateButtons(importState, mediaState, linkState, styleState, ruleState, removeState)
{
  if (!gDialog.expertMode) {
    importState = false;
    mediaState = false;
    linkState = false;
    styleState = false;
    ruleState = true;
  }
  EnableUI(gDialog.atimportButton, importState);
  EnableUI(gDialog.atmediaButton, mediaState);
  EnableUI(gDialog.linkButton, linkState);
  EnableUI(gDialog.styleButton, styleState);
  EnableUI(gDialog.ruleButton, ruleState);
  EnableUI(gDialog.removeButton, removeState);
  EnableUI(gDialog.upButton, removeState);
  EnableUI(gDialog.downButton, removeState);
}

function AddSingleButtonToInfobox(rows, label, callback)
{
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "center");
  var spacer = document.createElementNS(XUL_NS, "spacer");
  var hbox = document.createElementNS(XUL_NS, "hbox");
  var button = document.createElementNS(XUL_NS, "button");
  button.setAttribute("label", label);
  button.setAttribute("class", "align-right");
  button.setAttribute("oncommand", callback+"();");
  row.appendChild(spacer);
  hbox.appendChild(button);
  row.appendChild(hbox);
  rows.appendChild(row);
}

function AddEditableZoneToInfobox(rows, label, value, callback, focus)
{
  var labelLabel = document.createElementNS(XUL_NS, "label");
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "center");
  labelLabel.setAttribute("value", label);
  row.appendChild(labelLabel);

  var textbox = document.createElementNS(XUL_NS, "textbox");
  if (callback != "") 
    textbox.setAttribute("oninput", callback+"(this)");
  textbox.setAttribute("value", value);
  textbox.setAttribute("size", 20);
  row.appendChild(textbox);
  rows.appendChild(row);
  if (focus)
    SetTextboxFocus(textbox);
}

function AddRadioGroupToInfoBox(rows, label)
{
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "baseline");

  var labelLabel = document.createElementNS(XUL_NS, "label");
  labelLabel.setAttribute("value", label);
  row.appendChild(labelLabel);

  var radiogroup = document.createElementNS(XUL_NS, "radiogroup");
  row.appendChild(radiogroup);
  rows.appendChild(row);
  return radiogroup;
}

function AddRadioToRadioGroup(radiogroup, label, callback, selectorType, selected)
{
  var radio = document.createElementNS(XUL_NS, "radio");
  radio.setAttribute("label",     label);
  radio.setAttribute("oncommand", callback + "(" + selectorType + ");" );
  if (selected)
    radio.setAttribute("selected", "true");
  radiogroup.appendChild(radio);
}

function AddCheckboxToInfobox(rows, label, checkboxLabel, value, callback)
{
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "center");

  var labelLabel = document.createElementNS(XUL_NS, "label");
  labelLabel.setAttribute("value", label);
  row.appendChild(labelLabel);

  var checkbox = document.createElementNS(XUL_NS, "checkbox");
  checkbox.setAttribute("label", checkboxLabel);
  checkbox.setAttribute("checked", value);
  checkbox.setAttribute("oncommand", callback+"()");
  row.appendChild(checkbox);

  rows.appendChild(row);
}

function AddLabelToInfobox(rows, label, value, strong)
{
  var labelLabel = document.createElementNS(XUL_NS, "label");
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "center");
  labelLabel.setAttribute("value", label);
  if (strong) {
    labelLabel.setAttribute("class", "titleLabel");
  }
  row.appendChild(labelLabel);

  if (value) {
    var valueLabel = document.createElementNS(XUL_NS, "label");
    valueLabel.setAttribute("value", value);
    if (strong) {
      valueLabel.setAttribute("class", "titleLabel");
    }
    row.appendChild(valueLabel);
  }
  rows.appendChild(row);
}

function AddDeclarationToInfobox(rows, label, value, importance)
{
  var labelLabel = document.createElementNS(XUL_NS, "label");
  var row = document.createElementNS(XUL_NS, "row");
  row.setAttribute("align", "center");
  labelLabel.setAttribute("value", label);
  row.appendChild(labelLabel);

  var valueLabel = document.createElementNS(XUL_NS, "label");
  valueLabel.setAttribute("value", value);
  row.appendChild(valueLabel);

  var importanceLabel = document.createElementNS(XUL_NS, "label");
  importanceLabel.setAttribute("value", importance);
  if (importance == "!important") {
    importanceLabel.setAttribute("class", "bangImportant");
  }
  row.appendChild(importanceLabel);

  rows.appendChild(row);
}

function getSelectedItem(tree)
{
  if (tree.treeBoxObject.selection.count == 1)
    return tree.contentView.getItemAtIndex(tree.currentIndex);
  else
    return null;
}

function GetDeclarationText(styleRule, index)
{
  var pName = styleRule.style.item(index);
  return pName + ": " + styleRule.style.getPropertyValue(pName);
}

function GetDeclarationImportance(styleRule, index)
{
  var pName = styleRule.style.item(index);
  return styleRule.style.getPropertyPriority(pName);
}

function GetSheetContainer()
{
  var index = gDialog.selectedIndex;
  while (index >= 0 && objectsArray[index].type != SHEET) {
    index--;
  }
  return index;
}

function SetModifiedFlagOnStylesheet()
{
  var index = GetSheetContainer();
  if (index != -1) {
    objectsArray[index].modified = true;
    gDialog.modified = true;
  }
}

function PrepareInfoGridForCreation()
{
  gDialog.sheetTabbox.selectedTab = gDialog.sheetInfoTab;
  
  var gridrows = document.getElementById("sheetInfoTabGridRows");
  var grid     = document.getElementById("sheetInfoTabGrid");

  if (gridrows) {
    gridrows.parentNode.removeChild(gridrows);
  }

  gridrows = document.createElementNS(XUL_NS, "rows");
  gridrows.setAttribute("id", "sheetInfoTabGridRows");
  grid.appendChild(gridrows);
  grid.removeAttribute("style");
  return gridrows;
}

function CreateNewAtimportRule()
{
  var gridrows = PrepareInfoGridForCreation();

  gDialog.newType      = IMPORT_RULE;
  gDialog.newMediaList = "";
  gDialog.newURL       = "";
  gDialog.sheetInfoTabPanelTitle.setAttribute("value", "New Import At-rule");
  AddEditableZoneToInfobox(gridrows, "URL:",        gDialog.newURL,    "onNewURLChange", true);
  AddEditableZoneToInfobox(gridrows, "Media list:", gDialog.newMediaList, "onNewMediaListChange", false);
  AddSingleButtonToInfobox(gridrows, "Create Import At-rule", "onConfirmCreateNewObject", false);
}

function CreateNewStyleRule()
{
  var gridrows = PrepareInfoGridForCreation();

  gDialog.newExternal  = false;
  gDialog.newType      = STYLE_RULE;
  gDialog.newSelector  = "";
  gDialog.sheetInfoTabPanelTitle.setAttribute("value", "New Style Rule");

  var radiogroup = AddRadioGroupToInfoBox(gridrows, "Create a new:");
  AddRadioToRadioGroup(radiogroup, "named style (enter class name below)",
                       "onCreationStyleRuleTypeChange", CLASS_SELECTOR, true);
  AddRadioToRadioGroup(radiogroup, "style applied to all elements of type (enter type below)",
                       "onCreationStyleRuleTypeChange", TYPE_ELEMENT_SELECTOR, false);
  if (gDialog.expertMode) {
    AddRadioToRadioGroup(radiogroup, "style applied to all elements matching the following selector",
                         "onCreationStyleRuleTypeChange", GENERIC_SELECTOR, false);
    gDialog.newSelectorType = GENERIC_SELECTOR;
  }
  else {
    gDialog.newSelectorType = CLASS_SELECTOR;
  }
  AddEditableZoneToInfobox(gridrows, " ", gDialog.newSelector,  "onNewSelectorChange", true);

  AddSingleButtonToInfobox(gridrows, "Create Style Rule", "onConfirmCreateNewObject");
}

function onCreationStyleRuleTypeChange(type)
{
  gDialog.newSelectorType = type;
}

function CreateNewStyleElement()
{
  var gridrows = PrepareInfoGridForCreation();

  gDialog.newExternal  = false;
  gDialog.newType      = SHEET;
  gDialog.newMediaList = "";
  gDialog.newTitle     = "";
  gDialog.sheetInfoTabPanelTitle.setAttribute("value", "New Stylesheet");
  AddLabelToInfobox(gridrows, "Type:", "text/css", false);
  AddEditableZoneToInfobox(gridrows, "Media list:", gDialog.newMediaList, "onNewMediaListChange", true);
  AddEditableZoneToInfobox(gridrows, "Title:",      gDialog.newTitle,  "onNewTitleChange", false);

  AddSingleButtonToInfobox(gridrows, "Create Stylesheet", "onConfirmCreateNewObject")
}

function CreateNewLinkedSheet()
{
  var gridrows = PrepareInfoGridForCreation();

  gDialog.newExternal  = true;
  gDialog.newType      = SHEET;
  gDialog.newAlternate = false;
  gDialog.newURL       = "";
  gDialog.newMediaList = "";
  gDialog.newTitle     = "";
  gDialog.sheetInfoTabPanelTitle.setAttribute("value", "New Linked Stylesheet");
  AddLabelToInfobox(gridrows, "Type:", "text/css", false);
  AddCheckboxToInfobox(gridrows, "Alternate:", "check to create alternate stylesheet", gDialog.newAlternate,
                       "onNewAlternateChange");
  AddEditableZoneToInfobox(gridrows, "URL:",        gDialog.newURL,    "onNewURLChange", true);
  AddEditableZoneToInfobox(gridrows, "Media list:", gDialog.newMediaList, "onNewMediaListChange", false);
  AddEditableZoneToInfobox(gridrows, "Title:",      gDialog.newTitle,  "onNewTitleChange", false);

  AddSingleButtonToInfobox(gridrows, "Create Stylesheet", "onConfirmCreateNewObject")
  AddLabelToInfobox(gridrows, "", "(use Refresh button if stylesheet is not immediately downloaded)", false);
}

function Restart()
{
  var l = objectsArray.length;
  for (var i=0; i<l; i++) {
    delete objectsArray[i];
  }
  delete objectsArray;
  var gridrows = document.getElementById("sheetInfoTabGridRows");
  if (gridrows) {
    var elt = gridrows.lastChild;
    while (elt) {
      var tmp = elt.previousSibling;
      gridrows.removeChild(elt);
      elt = tmp;
    }
  }
  gDialog.selectedTab = GENERAL_TAB;
  gDialog.sheetInfoTabPanelTitle.setAttribute("value", "");
  InitSheetsTree(gDialog.sheetsTreechildren);
  UpdateButtons(false, false, true, true, false, false);
}

function Refresh()
{
  var index = gDialog.selectedIndex;
  Restart();
  if (index != -1)
    selectTreeItem(objectsArray[index].xulElt);
}

/* CALLBACKS */

function onNewAlternateChange()
{
  gDialog.newAlternate = !gDialog.newAlternate;
}

function onNewURLChange(elt)
{
  gDialog.newURL = elt.value;
}
  
function onNewSelectorChange(elt)
{
  gDialog.newSelector = elt.value;
}
  
function onNewMediaListChange(elt)
{
  gDialog.newMediaList = elt.value;
}

function onNewTitleChange(elt)
{
  gDialog.newTitle = elt.value;
}

function onStylesheetDisabledChange()
{
  gDialog.selectedObject.disabled = !gDialog.selectedObject.disabled;
}

function onStylesheetTitleChange(elt)
{
  var titleText = elt.value;
  gDialog.selectedObject.ownerNode.setAttribute("title", titleText);
  SetModifiedFlagOnStylesheet();
}

function onStylesheetMediaChange(elt)
{
  var mediaText= elt.value;
  gDialog.selectedObject.ownerNode.setAttribute("media", mediaText);
  SetModifiedFlagOnStylesheet();
}





function onConfirmCreateNewObject()
{
  /* do we have to create a stylesheet in non expert mode ? */
  gDialog.modified = true;
  var selector;
  if (gDialog.newType == STYLE_RULE) {
    var indexLastEmbeddedStylesheet = -1;
    for (var i = objectsArray.length-1; i >= 0 ; i--) {
      if (objectsArray[i].type == SHEET && ! objectsArray[i].external) {
        indexLastEmbeddedStylesheet = i;
        break;
      }
    }
    if (indexLastEmbeddedStylesheet != -1) {
      gDialog.selectedIndex = indexLastEmbeddedStylesheet;
    }
    else {
      gDialog.newExternal  = false;
      gDialog.newMediaList = "";
      gDialog.newTitle     = "";
      gDialog.newType      = SHEET;
      var selectorType     = gDialog.newSelectorType;
      selector             = gDialog.newSelector;
      onConfirmCreateNewObject();
      gDialog.newType         = STYLE_RULE;
      gDialog.newSelectorType = selectorType;
      gDialog.newSelector     = selector;
    }
  }
  var containerIndex;
  var cssObject;
  var l;
  var ruleIndex;
  var newSheetOwnerNode;
  var headNode;
  switch (gDialog.newType) {
    case STYLE_RULE:
      if (gDialog.newSelector != "") {
        containerIndex = gDialog.selectedIndex;
        while (objectsArray[containerIndex].type != SHEET &&
               objectsArray[containerIndex].type != MEDIA_RULE)
          containerIndex--;
        cssObject = objectsArray[containerIndex].cssElt;
        l = cssObject.cssRules.length;
        switch (gDialog.newSelectorType) {
          case TYPE_ELEMENT_SELECTOR:
          case GENERIC_SELECTOR:
            selector = gDialog.newSelector;
            break;
          case CLASS_SELECTOR:
            selector = "." + gDialog.newSelector;
            break;
        }
        cssObject.insertRule(selector + " { }", l);
        /* find inserted rule's index in objectsArray */
        var depth = objectsArray[containerIndex].depth;
        ruleIndex = containerIndex + 1;
        while (ruleIndex < objectsArray.length &&
               objectsArray[ruleIndex].depth > depth) {
          ruleIndex++;
        }
        Restart();
        selectTreeItem(objectsArray[ruleIndex].xulElt);
        objectsArray[containerIndex].modified = true;
      }
      break;
    case IMPORT_RULE:
      if (gDialog.newURL != "") {
        containerIndex = gDialog.selectedIndex;
        while (objectsArray[containerIndex].type != SHEET &&
               objectsArray[containerIndex].type != MEDIA_RULE)
          containerIndex--;
        cssObject = objectsArray[containerIndex].cssElt;
        l = cssObject.cssRules.length;
        ruleIndex = containerIndex + 1;
        while (ruleIndex < objectsArray.length &&
               (objectsArray[ruleIndex].type == CHARSET_RULE ||
                objectsArray[ruleIndex].type == IMPORT_RULE))
          ruleIndex++;
        cssObject.insertRule('@import url("'+ gDialog.newURL + '") ' + gDialog.newMediaList + ";",
                             ruleIndex - containerIndex - 1);
        FlushChanges();
        Restart();
        selectTreeItem(objectsArray[ruleIndex].xulElt);
        objectsArray[containerIndex].modified = true;
      }
      break;
    case SHEET:
      if (gDialog.newExternal && gDialog.newURL != "") {
        newSheetOwnerNode = editorShell.editorDocument.createElement("link");
        newSheetOwnerNode.setAttribute("type", "text/css");
        newSheetOwnerNode.setAttribute("href", gDialog.newURL);
        if (gDialog.newAlternate) {
          newSheetOwnerNode.setAttribute("rel", "alternate stylesheet");
        }
        else {
          newSheetOwnerNode.setAttribute("rel", "stylesheet");
        }
        if (gDialog.newMediaList != "") {
          newSheetOwnerNode.setAttribute("media", gDialog.newMediaList);
        }
        if (gDialog.newTitle != "") {
          newSheetOwnerNode.setAttribute("title", gDialog.newTitle);
        }
        headNode = GetHeadElement();
        headNode.appendChild(newSheetOwnerNode);

        if (!newSheetOwnerNode.sheet) {
          /* complex hack due to asynchronous load of external stylesheet */        
          var subtreeitem  = document.createElementNS(XUL_NS, "treeitem");
          var subtreerow   = document.createElementNS(XUL_NS, "treerow");
          var subtreecell  = document.createElementNS(XUL_NS, "treecell");
          subtreecell.setAttribute("label", gDialog.newURL);
          var o = newObject( subtreeitem, true, OWNER_NODE, newSheetOwnerNode, false, 0 );
          objectsArray.push(o);
          subtreeitem.setAttribute("container", "true");
          subtreecell.setAttribute("properties", "external");
          subtreerow.appendChild(subtreecell);
          subtreeitem.appendChild(subtreerow);
          gDialog.sheetsTreechildren.appendChild(subtreeitem);
        }
        else {
          var index = objectsArray.length;
          Restart();
          selectTreeItem(objectsArray[index].xulElt);
          objectsArray[index].modified = true;
        }
      }
      else if (!gDialog.newExternal) {
        newSheetOwnerNode = editorShell.editorDocument.createElement("style");
        newSheetOwnerNode.setAttribute("type", "text/css");
        if (gDialog.newMediaList != "") {
          newSheetOwnerNode.setAttribute("media", gDialog.newMediaList);
        }
        if (gDialog.newTitle != "") {
          newSheetOwnerNode.setAttribute("title", gDialog.newTitle);
        }
        headNode = GetHeadElement();
        headNode.appendChild(newSheetOwnerNode);
        FlushChanges();
        Restart();
        selectTreeItem(objectsArray[objectsArray.length - 1].xulElt);
      }
      break;
  }
}

/****************************************************************/
/* Selects treeitem |aItem| in its tree container               */
/****************************************************************/
function selectTreeItem(aItem)
{
  /* first make sure item's containers are open */
  var tmp = aItem.parentNode;
  while (tmp && tmp.nodeName != "tree") {
    if (tmp.nodeName == "treeitem")
      tmp.setAttribute("open", "true");
    tmp = tmp.parentNode;
  }

  /* then select the item */
  var itemIndex = gDialog.sheetsTree.contentView.getIndexOfItem(aItem);
  gDialog.sheetsTree.treeBoxObject.selection.select(itemIndex);
  /* and make sure it is visible in the clipping area of the tree */
  gDialog.sheetsTree.treeBoxObject.ensureRowIsVisible(itemIndex);
}

function getRuleIndexInRulesList(rulelist, rule)
{
  if (!rule || !rulelist)
    return -1;

  var i;
  for (i=0; i<rulelist.length; i++) {
    if (rulelist.item(i) == rule) {
      return i;
    }
  }
  return -1;
}

function FindObjectIndexInObjectsArray(object)
{
  var i, l = objectsArray.length;
  for (i=0; i<l; i++)
    if (objectsArray[i].cssElt == object)
      return i;
  return -1;
}

function RemoveObject()
{
  var objectIndex = FindObjectIndexInObjectsArray(gDialog.selectedObject);
  if (objectIndex == -1) return;

  var ruleIndex;
  switch (gDialog.selectedType) {
    case SHEET:
      var ownerNode = gDialog.selectedObject.ownerNode;
      ownerNode.parentNode.removeChild(ownerNode);
      Restart();
      if (objectIndex < objectsArray.length )
        selectTreeItem(objectsArray[objectIndex].xulElt);
      break;
    case IMPORT_RULE:
      var ruleIndexInTree = FindObjectIndexInObjectsArray(gDialog.selectedObject.parentStyleSheet);
      objectsArray[ruleIndexInTree].modified = true;
      ruleIndex = getRuleIndexInRulesList(gDialog.selectedObject.parentStyleSheet.cssRules,
                                          gDialog.selectedObject);
      if (ruleIndex != -1) {
        gDialog.selectedObject.parentStyleSheet.deleteRule(ruleIndex);
      }
      Restart();
      objectsArray[ruleIndexInTree].modified = true;
      if (objectIndex < objectsArray.length )
        selectTreeItem(objectsArray[objectIndex].xulElt);
      break;
    case STYLE_RULE:
      var ruleIndexInTree = FindObjectIndexInObjectsArray(gDialog.selectedObject.parentStyleSheet);
      objectsArray[ruleIndexInTree].modified = true;
      if (gDialog.selectedObject.parentRule) {
        /* this style rule is contained in an at-rule */
        /* need to remove the rule only from the at-rule listing it */
        ruleIndex = getRuleIndexInRulesList(gDialog.selectedObject.parentRule.cssRules,
                                            gDialog.selectedObject);
        if (ruleIndex != -1) {
          gDialog.selectedObject.parentRule.deleteRule(ruleIndex);
        }
      }
      else {
        ruleIndex = getRuleIndexInRulesList(gDialog.selectedObject.parentStyleSheet.cssRules,
                                            gDialog.selectedObject);
        if (ruleIndex != -1) {
          gDialog.selectedObject.parentStyleSheet.deleteRule(ruleIndex);
        }
      }
      objectsArray[objectIndex].xulElt.parentNode.removeChild(objectsArray[objectIndex].xulElt);
      objectsArray.splice(objectIndex, 1);
      selectTreeItem(objectsArray[Math.min(objectIndex, objectsArray.length - 1)].xulElt);
      break;
  }
}

function MoveObjectUp()
{
  var objectIndex = FindObjectIndexInObjectsArray(gDialog.selectedObject);
  if (objectIndex == -1) return;
}

function MoveObjectDown()
{
  var objectIndex = FindObjectIndexInObjectsArray(gDialog.selectedObject);
  if (objectIndex == -1) return;
}

