/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CaScadeS, a stylesheet editor for Composer.
 *
 * The Initial Developer of the Original Code is
 * Daniel Glazman.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Original author: Daniel Glazman <daniel@glazman.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function openCascadesDialog()
{
  window.openDialog("chrome://cascades/content/EdCssProps.xul","_blank", "chrome,close,modal,titlebar");
  window._content.focus();
}

function openSpecificDialog(selector, cssDeclarations, tabs, title, tagName)
{
  var selectedBlock = getSelectedBlock();
  if (!selectedBlock) return;
  // var htmlEditor = editorShell.editor.QueryInterface(Components.interfaces.nsIHTMLEditor);
  //htmlEditor.selectElement(selectedBlock);
  window.openDialog("chrome://cascades/content/specificCssProps.xul","_blank", "chrome,close,modal,titlebar",
                    selector, cssDeclarations, tabs, title, tagName);
  window._content.focus();
}

function openSpecificDialogNoCheck(selector, cssDeclarations, tabs, title, tagName)
{
  window.openDialog("chrome://cascades/content/specificCssProps.xul","_blank", "chrome,close,modal,titlebar",
                    selector, cssDeclarations, tabs, title, tagName);
  window._content.focus();
}

const blockDpdtMenuitems = [ "backgroundMenuitem",
                             "bordersMenuitem",
                             "boxMenuitem",
                             "marginsMenuitem",
                             "droppedEditMenuitem",
                             "droppedRemoveMenuitem",
                             "firstlineEditMenuitem",
                             "firstlineRemoveMenuitem" ];

function InitStyleMenu()
{
  var selectedBlock = getSelectedBlock();
  for (var i=0; i<blockDpdtMenuitems.length; i++) {
    var elt = document.getElementById(blockDpdtMenuitems[i]);
    if (elt) {
      if (selectedBlock) {
        elt.removeAttribute("disabled");
      }
      else {
        elt.setAttribute("disabled", "true");
      }
    }
  }
}

function getSelectedBlock()
{
  var anchorNode = editorShell.editorSelection.anchorNode;
  if (!anchorNode) return null;
  var node;
  if (anchorNode.firstChild)
  {
    // Start at actual selected node
    var offset = editorShell.editorSelection.anchorOffset;
    // Note: If collapsed, offset points to element AFTER caret,
    //  thus node may be null
    node = anchorNode.childNodes.item(offset);
  }
  if (!node)
    node = anchorNode;

  while (!nodeIsBlock(node) && node.nodeName.toLowerCase() != "body") {
    node = node.parentNode;
  }
  if (node.nodeName.toLowerCase() == "body") {
    node = null;
  }
  return node;
}

function nodeIsBlock(node)
{
  // HR doesn't count because it's not a container
  return !node || (node.localName != 'HR' && editorShell.NodeIsBlock(node));
}

function removeRuleFor(selector)
{
  var elt = getSelectedBlock();
  if (!elt) return;
  var id = elt.getAttribute("id");
  if (!id || id == "") return;
  var sst = editorShell.editorDocument.styleSheets;
  var modified = false;
  if (sst) {
    var ssn = sst.length;
    if (ssn) {
      var i;
      for (i=0; i<ssn; i++) {
        var ownerNode = sst[i].ownerNode;
        var ownerTag = ownerNode.nodeName.toLowerCase();
        if (ownerTag == "style") {
          modified = false;
          var rules = sst[i].cssRules;
          for (j=rules.length - 1; j>=0 ; j--) {
            if (rules[j].type == CSSRule.STYLE_RULE) {
              if (rules[j].selectorText == "#"+id+selector) {
                sst[i].deleteRule(j);
                modified = true;
              }
            }
          }
          if (modified) {
            /* the stylesheet has changed, need to serialize it */
            var cssText = "\n";
            var l = sst[i].cssRules.length;
            for (var j = 0; j< l; j++) {
              var tmp = sst[i].cssRules.item(j).cssText;
              var re = /;\s*/;
              var cssTextSplitted = tmp.split(re);
              cssText += "  ";
              for (var k=0; k<cssTextSplitted.length; k++) {
                cssText += cssTextSplitted[k];
                if (k != cssTextSplitted.length-1)
                  cssText += ";\n    ";
              }
              if (l > 1) cssText += "\n\n";
            }
            var child = ownerNode.lastChild;
            while (child) {
              var tmp = child.previousSibling;
              ownerNode.removeChild(child);
              child = tmp;
            }
            var textNode = editorShell.editorDocument.createTextNode(cssText + "\n");
            ownerNode.appendChild(textNode);
          }
        }
      }
    }
  }
  if (modified) {
    editorShell.editor.incrementModificationCount(1);
  }
}

function OpenAbout()
{
  window.openDialog("chrome://cascades/content/aboutCascades.xul","_blank", "chrome,close,modal,titlebar");
  //window._content.focus();
}
