/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CaScadeS, a stylesheet editor for Composer.
 *
 * The Initial Developer of the Original Code is
 * Daniel Glazman.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Original author: Daniel Glazman <daniel@glazman.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gTimerID;

const urlString = "url(";

const GENERAL_TAB    = 1;
const TEXT_TAB       = 2;
const BACKGROUND_TAB = 3;
const BORDER_TAB     = 4;
const BOX_TAB        = 5;
const AURAL_TAB      = 6;

var textColor, backgroundColor;
var gHaveDocumentUrl = false;
var predefSelector = "";

function doDump(text, value) {
  dump("===> " + text + " : " + value + "\n");
}

// dialog initialization code
function Startup()
{
  if (!InitEditorShell())
    return;

  var selector = "";
  var cssRulesText = "";
  var tabs = "all";
  var firstTab = null;
  var title = "Styles";
  var tagName = null;
  if ("arguments" in window && window.arguments.length >= 5) {
    selector     = window.arguments[0];
    cssRulesText = window.arguments[1];
    tabs         = window.arguments[2];
    title        = window.arguments[3];
    tagName      = window.arguments[4];
  }
  var w = document.getElementById("specificCssPropsWindow");
  w.setAttribute("title", title);

  // gDialog is declared in EdDialogCommon.js
  // Set commonly-used widgets like this:
  gDialog.textTab                       = document.getElementById("textTab");
  gDialog.brownFoxLabel                 = document.getElementById("brownFoxLabel");
  gDialog.backgroundImageInput          = document.getElementById("backgroundImageInput");
  gDialog.backgroundPreview             = document.getElementById("backgroundPreview");
  gDialog.sheetTabbox                   = document.getElementById("sheetTabbox");
  gDialog.backgroundColorInput          = document.getElementById("backgroundColorInput");
  gDialog.textColorInput                = document.getElementById("textColorInput");
  gDialog.backgroundRepeatMenulist      = document.getElementById("backgroundRepeatMenulist");
  gDialog.backgroundAttachmentCheckbox  = document.getElementById("backgroundAttachmentCheckbox");
  gDialog.xBackgroundPositionRadiogroup = document.getElementById("xBackgroundPositionRadiogroup");
  gDialog.yBackgroundPositionRadiogroup = document.getElementById("yBackgroundPositionRadiogroup");
  gDialog.fontFamilyRadiogroup          = document.getElementById("fontFamilyRadiogroup");
  gDialog.customFontFamilyInput         = document.getElementById("customFontFamilyInput");
  gDialog.predefFontFamilyMenulist      = document.getElementById("predefFontFamilyMenulist");
  gDialog.fontSizeInput                 = document.getElementById("fontSizeInput");

  gDialog.topBorderStyleMenulist        = document.getElementById("topBorderStyleMenulist");
  gDialog.topBorderWidthInput           = document.getElementById("topBorderWidthInput");
  gDialog.topBorderColorInput           = document.getElementById("topBorderColorInput");

  gDialog.leftBorderStyleMenulist       = document.getElementById("leftBorderStyleMenulist");
  gDialog.leftBorderWidthInput          = document.getElementById("leftBorderWidthInput");
  gDialog.leftBorderColorInput          = document.getElementById("leftBorderColorInput");

  gDialog.rightBorderStyleMenulist      = document.getElementById("rightBorderStyleMenulist");
  gDialog.rightBorderWidthInput         = document.getElementById("rightBorderWidthInput");
  gDialog.rightBorderColorInput         = document.getElementById("rightBorderColorInput");

  gDialog.bottomBorderStyleMenulist     = document.getElementById("bottomBorderStyleMenulist");
  gDialog.bottomBorderWidthInput        = document.getElementById("bottomBorderWidthInput");
  gDialog.bottomBorderColorInput        = document.getElementById("bottomBorderColorInput");

  gDialog.allFourBordersSame            = document.getElementById("allFourBordersSame");
  gDialog.borderPreview                 = document.getElementById("borderPreview");

  gDialog.modified = false;
  gDialog.selectedIndex = -1;

  if (tabs != "all") {
    firstTab = DisableTabIfNotSpecified("aural", tabs, firstTab);
    firstTab = DisableTabIfNotSpecified("box", tabs, firstTab);
    firstTab = DisableTabIfNotSpecified("border", tabs, firstTab);
    firstTab = DisableTabIfNotSpecified("background", tabs, firstTab);
    firstTab = DisableTabIfNotSpecified("text", tabs, firstTab);
    gDialog.sheetTabbox.selectedTab = document.getElementById( firstTab + "Tab" );
  }

  var elt = getSelectedBlock();
  if (!elt && !tagName) return;
  if (selector != null) {
    if (selector == "" || (selector.length > 0 && selector[0] == ':')) {
      var id = elt.getAttribute("id");
      if (!id || id == "") {
        var foo = 100000 * Math.random();
        id = "mozid" + Number(Math.round(foo)).toString();
        elt.setAttribute("id", id);
      }
      selector = "#" + id + selector;
    }
    var sst = editorShell.editorDocument.styleSheets;
    var needNewStyleSheet = true;
    var needNewRule       = true;
    var sheet = null;
    var l = -1;
    if (sst) {
      var ssn = sst.length;
      if (ssn) {
        /* do we already have a rule ? */
        var i;
        for (i=ssn-1; i>=0; i--) {
          var ownerNode = sst[i].ownerNode;
          var ownerTag = ownerNode.nodeName.toLowerCase();
          if (ownerTag == "style") {
            if (!sheet) {
              needNewStyleSheet = false;
              l = sst[i].length;
              sheet = sst[i];
            }
            var j;
            var rules = sst[i].cssRules;
            for (j=rules.length-1; j>=0; j--) {
              if (rules[j].type == CSSRule.STYLE_RULE) {
                if (rules[j].selectorText == selector) {
                  needNewRule = false;
                  gDialog.selectedObject = rules[j];
                  break;
                }
              }
            }
            if (!needNewRule) break;
          }
        }
      }
    }
    if (needNewRule) {
      if (needNewStyleSheet) {
        var newSheetOwnerNode = editorShell.editorDocument.createElement("style");
        newSheetOwnerNode.setAttribute("type", "text/css");
        var headNode = GetHeadElement();
        headNode.appendChild(newSheetOwnerNode);
        sst = editorShell.editorDocument.styleSheets;
        sheet = sst[sst.length - 1];
      }
      var l = sheet.cssRules.length;
      sheet.insertRule( selector+"{ "+cssRulesText+" }", l);
      gDialog.selectedObject = sheet.cssRules[l];
      gDialog.selectedSheet  = sheet;
    }
  }
  else {
    if (tagName != null) {
      var rangeCount = editorShell.editorSelection.rangeCount;
      if (rangeCount == 1) {
        var range = editorShell.editorSelection.getRangeAt(0);
        if (range.startContainer.nodeType == Node.TEXT_NODE)
          range.startContainer.splitText(range.startOffset);
        if (range.endContainer.nodeType == Node.TEXT_NODE)
          range.endContainer.splitText(range.endOffset);
        var range = editorShell.editorSelection.getRangeAt(0);
        var node = range.commonAncestorContainer;
        if (node.nodeName.toLowerCase() == "body" &&
            !(range.startContainer == range.endContainer
              && range.startOffset == range.endOffset)) {
          var nodeList = node.childNodes;
          var j = -1;
          for (var i=0; i<nodeList.length; i++) {
            if (range.isPointInRange(node, i)) {
              j = i;
              break;
            }
          }
          if (j != -1) {
            var k = j+1;
            if (j < nodeList.length-1) {
              for (var i=j+1;  i<nodeList.length && range.isPointInRange(node, i); i++) {
                k = i;
              }
            }
            nodeList = node.childNodes;
            var firstNode = nodeList[j];
            elt = editorShell.editorDocument.createElement(tagName);
            node.insertBefore(elt, firstNode);
            nodeList = node.childNodes;
            for (var i=k+1; i>=j+1; i--) {
              var foo = node.removeChild(nodeList[i]);
              elt.insertBefore(foo, elt.firstChild);
            }
          }
        }
        else {
          if (range.startContainer == range.endContainer
              && range.startOffset == range.endOffset) {
            // selection is a single node, not the body
            node = range.startContainer.childNodes[range.startOffset];
          }
          if (node.nodeName.toLowerCase() == tagName)
            elt = node;
          else {
            elt = editorShell.editorDocument.createElement(tagName);
            node.parentNode.insertBefore(elt, node);
            elt.appendChild(node);
          }
        }
      }
      else {
        window.alert("Selection cannot be turned into a leyer !");
        window.close();
        return;
      }
    }
    gDialog.selectedObject = elt;
    gDialog.selectedHasStyle = elt.hasAttribute("style");
    if (gDialog.selectedHasStyle)
      gDialog.selectedStyle = elt.getAttribute("style");
    elt.setAttribute("style", cssRulesText + ";" + gDialog.selectedStyle);
    gDialog.selectedSheet  = null;
  }
  InitDialog();
  
  // Set window location relative to parent window (based on persisted attributes)
  SetWindowLocation();

  // Set focus to first widget in dialog, e.g.:
}

function DisableTabIfNotSpecified(id, tabs, firstTab)
{
  if (tabs.indexOf(id) == -1) {
    /* disable the corresponding tab */
    var elt = document.getElementById( id + "Tab" );
    elt.setAttribute("collapsed", "true");
  }
  else
    firstTab = id;
  return firstTab;
}

function InitDialog()
{
  InitTextTabPanel();
  InitBackgroundTabPanel();
  InitBorderTabPanel();
  InitBoxTabPanel();
}

function getSelectedBlock()
{
  var anchorNode = editorShell.editorSelection.anchorNode;
  if (!anchorNode) return null;
  var node;
  if (anchorNode.firstChild)
  {
    // Start at actual selected node
    var offset = editorShell.editorSelection.anchorOffset;
    // Note: If collapsed, offset points to element AFTER caret,
    //  thus node may be null
    node = anchorNode.childNodes.item(offset);
  }
  if (!node)
    node = anchorNode;

  while (!nodeIsBlock(node) && node.nodeName.toLowerCase() != "body") {
    node = node.parentNode;
  }
  if (node.nodeName.toLowerCase() == "body") {
    node = null;
  }
  return node;
}

function nodeIsBlock(node)
{
  // HR doesn't count because it's not a container
  return !node || (node.localName != 'HR' && editorShell.NodeIsBlock(node));
}

function SetModifiedFlagOnStylesheet()
{
  /* STUB */
}

function FlushChanges()
{
  var sheet = gDialog.selectedSheet;
  if (sheet) {
    SerializeEmbeddedSheet(sheet);
  }
  editorShell.editor.incrementModificationCount(1);
}

function CancelChanges()
{
  var sheet = gDialog.selectedSheet;
  if (sheet) {
    sheet.deleteRule(sheet.cssRules.length - 1);
  }
  else {
    if (gDialog.selectedHasStyle)
      gDialog.selectedObject.setAttribute("style", gDialog.selectedStyle);
    else
      gDialog.selectedObject.removeAttribute("style");
  }
}
